#!/usr/bin/env python3

import argparse, os, sys, time, copy, redis
from dnslib import RR
from dnslib.server import DNSServer, DNSHandler, BaseResolver, DNSLogger, RCODE, QTYPE
from abuse import Abuse

class FixedResolver(BaseResolver):
	def __init__(self, r=None, domain='abuse-contact.findabuse.email'):
		self.domain = domain
		self.r = r

	def resolve(self, request, handler):
		reply = request.reply()
		qname = str(request.q.qname).replace('.{}'.format(self.domain), '')
		qtype = QTYPE[request.q.qtype]

		if qtype in ['TXT']:
			if self.r.get(str(qname)[:-1]):
				contacts = self.r.get(str(qname)[:-1]).decode("utf-8").split(',')
			else:
				contacts = Abuse(str(qname)[:-1]).lookup()
				self.r.set(str(qname)[:-1], ",".join(contacts))

			rrs = RR.fromZone(". 3600 IN TXT {}".format(",".join(contacts)))
			for r in rrs:
				a = copy.copy(r)
				a.rname = qname
				reply.add_answer(a)
		else:
			reply.header.rcode = getattr(RCODE,'NXDOMAIN')
		return reply

def main():
	p = argparse.ArgumentParser(description="findabuse.email: DNS resolver")

	# redis server settings
	p.add_argument("--redis", "-r",
		default=os.getenv("REDIS_URL", ""),
	)

	# dns server settings
	p.add_argument("--port", "-p",
		type=int,
		default=53,
		metavar="<port>",
		help="Server port (default:53)"
	)
	p.add_argument("--address", "-a",
		default="",
		metavar="<address>",
		help="Listen address (default:all)"
	)
	p.add_argument("--tcp",
		action='store_true',
		default=False,
		help="TCP server (default: UDP only)"
	)
	p.add_argument("--udplen", "-u",
		type=int,default=0,
		metavar="<udplen>",
		help="Max UDP packet length (default:0)"
	)
	p.add_argument("--log", 
		default="request,reply,truncated,error",
		help="Log hooks to enable (default: +request,+reply,+truncated,+error,-recv,-send,-data)"
	)
	p.add_argument("--log-prefix",
		action='store_true',
		default=False,
		help="Log prefix (timestamp/handler/resolver) (default: False)"
	)
	args = p.parse_args()

	if not args.redis == "":
		r = redis.from_url(args.redis)

	resolver = FixedResolver(r)
	logger = DNSLogger(args.log,args.log_prefix)

	print("Starting Fixed Resolver (%s:%d) [%s]" % (
						args.address or "*",
						args.port,
						"UDP/TCP" if args.tcp else "UDP"))

	if args.udplen:
		DNSHandler.udplen = args.udplen

	udp_server = DNSServer(resolver,
						port=args.port,
						address=args.address,
						logger=logger)
	udp_server.start_thread()

	if args.tcp:
		tcp_server = DNSServer(resolver,
							port=args.port,
							address=args.address,
							tcp=True,
							logger=logger)
		tcp_server.start_thread()

	while udp_server.isAlive():
		time.sleep(1)

if __name__ == '__main__':
	main()



#!/usr/bin/env python3

import argparse, os, sys, json
from abuse import Abuse
from util import Utils

def main():
	p = argparse.ArgumentParser(description="findabuse.email: CLI")

	p.add_argument("address", nargs=1, help="the IP address to lookup")
	p.add_argument("--json", default=False, action="store_true")

	args = p.parse_args()

	# Validation
	try:
		ip = Utils(args.address[0])
	except ValueError:
		print("The address you provided, {}, is not a valid IPv4 / IPv6 address".format(args.address[0]))
		exit()

	# So invalid data gone
	try:
		contacts = Abuse(args.address[0]).lookup()
		if args.json:
			print(json.dumps({
				'results': contacts
				})
			)
		else:
			for c in contacts:
				print(c)
	except:
		print("We encountered an error while processing your request")
		exit()

if __name__ == '__main__':
	main()


#!/usr/bin/env python3

import argparse, os, sys, time, copy, redis, json
from bottle import route, request, response, redirect, default_app, view, template, static_file
from abuse import Abuse

def set_content_type(fn):
	def _return_json(*args, **kwargs):
		response.headers['Content-Type'] = 'application/json'
		if request.method != 'OPTIONS':
			return fn(*args, **kwargs)
	return _return_json

def enable_cors(fn):
	def _enable_cors(*args, **kwargs):
		response.headers['Access-Control-Allow-Origin'] = '*'
		response.headers['Access-Control-Allow-Methods'] = 'GET, POST, PUT, OPTIONS'
		response.headers['Access-Control-Allow-Headers'] = 'Origin, Accept, Content-Type, X-Requested-With, X-CSRF-Token'

		if request.method != 'OPTIONS':
			return fn(*args, **kwargs)
	return _enable_cors

@route('/api/v1/<ip>', method=('OPTIONS', 'GET'))
@enable_cors
@set_content_type
def lookup(ip):
	global r

	output = {
		"results": {},
		"results_info": {}
	}

	addresses = ip.split(',')

	for a in addresses:
		addr = a.strip()

		if r.get(addr):
			contacts = r.get(addr).decode('utf-8').split(',')
		else:
			contacts = Abuse(addr).lookup()
			r.set(addr, ",".join(contacts))

		output['results'][addr] = {
			'contacts': contacts
		}

	output['results_info']['count'] = len(output['results'])
	return json.dumps(output)

def main():
	p = argparse.ArgumentParser(description="findabuse.email: API Server")

	# redis server settings
	p.add_argument("--redis", "-r",
		default=os.getenv("REDIS_URL", ""),
	)

	# dns server settings
	p.add_argument("--port", "-p",
		type=int,
		default=os.getenv("PORT", 5000),
		metavar="<port>",
		help="Server port (default: 5000)"
	)
	p.add_argument("--address", "-a",
		default=os.getenv("HOST", "127.0.0.1"),
		metavar="<address>",
		help="Listen address (default: localhost)"
	)
	args = p.parse_args()

	if not args.redis == "":
		global r
		r = redis.from_url(args.redis)

	try:
		app = default_app()
		app.run(host=args.address, port=args.port, server='tornado')
	except:
		print("Unable to start server on {}:{}".format(args.address, args.port))
		exit()

if __name__ == '__main__':
	main()


#!/usr/bin/env python3

from providers import Abusix, BGPView, NoticeAndAction
from collections import Counter

class Abuse:
	def __init__(self, address):
		self.address = address

	def sort(self, data):
		contacts = []
		for contact, count in Counter(data).most_common():
			contacts.append(contact)
		return list(set(filter(None, contacts)))

	def remove(self, data):
		contacts = []
		blacklist = [
			'apnic.net'
		]
		for contact in data:
			if not contact in blacklist and not contact.split('@')[1] in blacklist:
				contacts.append(contact)
		return contacts

	def lookup(self):
		abusix = Abusix(self.address).lookup()
		bgpview = BGPView(self.address).lookup()
		notice = NoticeAndAction(self.address).lookup()

		contacts = abusix + bgpview + notice
		return self.remove(self.sort(contacts))